#!/usr/bin/python3

# Version 0.2.4
# Date update: 23.05.2024
# Email: aleksandr.egai@red-soft.ru
# (c) RED SOFT


import configparser
import sys
sys.path.insert(0, '/usr/share/connectfolder/')
import ui_connectfolder  # pyuic5 -x ui_connectfolder.ui -o ui_connectfolder.py
import re
import subprocess
import os
from PyQt5.QtWidgets import *
from PyQt5 import QtWidgets, QtCore, QtGui
from PyQt5.QtGui import QRegExpValidator
from PyQt5.QtCore import QRegExp
from sys import argv

p_folder = '/root/.auth_smb'
p_autosamba = '/etc/auto.samba'
p_automaster = '/etc/auto.master'
parm_smbver1 = 'vers=1.0,'
params_automaster = '/media/share    /etc/auto.samba    --ghost'
config = configparser.ConfigParser()
delete_auth_file = ''
delete_p_autosamba_folder = ''
line_number = 0


def is_in_domain():
    cmd = "realm list"
    try:
        result = os.popen(cmd).read()
        if result.strip() == "":
            return False
        else:
            return True
    except:
        return False


class ConnectfolderApp(QtWidgets.QMainWindow, ui_connectfolder.Ui_MainWindow):
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.setWindowFlags(
            QtCore.Qt.Window |
            QtCore.Qt.CustomizeWindowHint |
            QtCore.Qt.WindowTitleHint |
            QtCore.Qt.WindowCloseButtonHint |
            QtCore.Qt.WindowStaysOnTopHint |
            QtCore.Qt.WindowMinimizeButtonHint
        )
        self.setWindowTitle("ConnectFolder-0.2.4")
        self.pushButton_delete.setVisible(False)
        self.setMaximumSize(self.width(), self.height())
        self.setMinimumSize(self.size())
#        self.radioButton_autofs.setChecked(True)
        self.radioButton_autofs.setVisible(False)
        self.pushButton_connect.clicked.connect(self.button_connect)
        self.pushButton_change_list.clicked.connect(self.button_change_list)
        self.pushButton_delete.clicked.connect(self.button_delete)
        self.checkBox_domain.clicked.connect(self.click_checkBox_domain)
        self.radioButton_gio.setVisible(False)
        self.label_check.clear()
        # Валидатор для lineEdit_folder
        reg_ex = QRegExp("[a-zA-Z0-9]+")  # разрешены буквы и цифры
        self.lineEdit_folder.setValidator(QRegExpValidator(reg_ex))
        # Валидатор для lineEdit_domain
        reg_ex = QRegExp("[a-zA-Z0-9.-]+")  # разрешены буквы, цифры, точка и дефис
        self.lineEdit_domain.setValidator(QRegExpValidator(reg_ex))

        # Добавляем переменную-флаг для отслеживания режима редактирования
        self.is_edit_mode = False

        # Создаем модель данных
        self.model = QtGui.QStandardItemModel()
        self.model.setRowCount(0)
        self.model.setColumnCount(2)
        self.tableView.setModel(self.model)
        self.tableView.setEditTriggers(QtWidgets.QTableView.NoEditTriggers)
        self.tableView.setSelectionBehavior(QAbstractItemView.SelectRows)  # Однократное выделение строк

        # Заголовки столбцов
        self.model.setHeaderData(0, QtCore.Qt.Horizontal, "Сетевой каталог")
        self.model.setHeaderData(1, QtCore.Qt.Horizontal, "Локальный каталог")

        # Подключаем сигнал clicked к слоту on_tableView_clicked
        self.tableView.clicked.connect(self.on_tableView_clicked)
        self.selected_row = None  # добавляем атрибут

        #  Загрузка '/etc/auto.samba' в TableView
        self.load_data_tableview()
        # Выравнивание столбцов
        table_width = self.tableView.width()
        column_width = int(table_width * 4.7)
        self.tableView.setColumnWidth(0, column_width)
        self.tableView.setColumnWidth(1, int(column_width / 2))

        if is_in_domain():
            print("Компьютер находится в домене.")
            self.checkBox_domain.setChecked(True)  # установить флажок
            self.lineEdit_password.setDisabled(True)
            self.lineEdit_password.setStyleSheet("background-color: #d9d9d9;")
            self.lineEdit_username.setDisabled(True)
            self.lineEdit_username.setStyleSheet("background-color: #d9d9d9;")
            self.lineEdit_domain.setDisabled(True)
            self.lineEdit_domain.setStyleSheet("background-color: #d9d9d9;")

        else:
            print("Компьютер не находится в домене.")
            self.checkBox_domain.setChecked(False)  # снять флажок

    @QtCore.pyqtSlot(QtCore.QModelIndex)
    def on_tableView_clicked(self, index):
        global delete_auth_file, delete_p_autosamba_folder, line_number
        # Получаем модель данных таблицы
        model = self.tableView.model()
        # Проверяем, что клик был по ячейке (а не по заголовку столбца)
        if index.isValid():
            # Получаем индексы всех ячеек строки
            row_index = index.row()
            line_number = row_index
            column_count = model.columnCount()
            start_index = model.index(row_index, 0)
            end_index = model.index(row_index, column_count - 1)
            selection = QtCore.QItemSelection(start_index, end_index)

            # Если строка не выделена, выделяем ее
            if index.row() != self.selected_row:
                # Выделяем всю строку
                self.tableView.selectionModel().select(selection, QtCore.QItemSelectionModel.Select)
                # Если нажата кнопка редактирования списка
                if self.is_edit_mode:
                    clear_lineEdit(self)
                    # Получаем значения всех ячеек строки
                    row_values = []
                    for col in range(model.columnCount()):
                        cell_value = model.data(model.index(index.row(), col))
                        row_values.append(cell_value)
                    self.lineEdit_path.setText(row_values[0])
                    self.lineEdit_folder.setText(row_values[1].replace('/media/share/', ''))
                    auth_path = p_folder + '/' + row_values[1].replace('/media/share/', '')
                    lineFile = open_auto_samba()
                    for i, line in enumerate(lineFile):
                        if 'vers=1.0' in line and i == line_number:
                            self.checkBox_smb1.setChecked(True)  # Установить флажок в состояние "включено"
                            break
                        else:
                            self.checkBox_smb1.setChecked(False)
                    if os.path.isfile(auth_path):
                        smb_user, smb_password, smb_domain = get_smbuser(auth_path)
                        self.lineEdit_username.setText(smb_user)
                        self.lineEdit_password.setText(smb_password)
                        self.lineEdit_domain.setText(smb_domain)
                        delete_auth_file = auth_path
                        delete_p_autosamba_folder = row_values[1].replace('/media/share/', '')
                        print(delete_auth_file, delete_p_autosamba_folder)
                else:
                    # Проверка доступности каталога
                    # Цикл для определения значения всех ячеек строки
                    row_values = []
                    for col in range(model.columnCount()):
                        cell_value = model.data(model.index(index.row(), col))
                        row_values.append(cell_value)

                    folder_path = row_values[1] + '/'
                    if len(argv) < 2:
                        user_arg = os.getlogin()
                    else:
                        user_arg = argv[1]

                    script = f"cd {folder_path} && cd /"
                    command = f"su \"{user_arg}\" bash -c \"{script}\""
                    result = os.system(command)

                    if result == 0:
                        self.label_check.setText('Каталог доступен!')
                        self.label_check.setStyleSheet("color: green;")
                    else:
                        self.label_check.setText('Каталог не доступен!')
                        self.label_check.setStyleSheet("color: red;")

    def load_data_tableview(self):
        # Очистка
        model = self.tableView.model()
        model.removeRows(0, model.rowCount())
        # читаем '/etc/auto.samba' для загрузки в TableView
        lineFile = open_auto_samba()
        for l in lineFile:
            if not l.startswith('#') and len(l.strip()) > 0:
                # Регулярное выражение для разбивки строки
                parts = re.split(r'\s*:\s*|,|\s+', l.strip())
                name = parts[0]
                if len(parts) > 5 and parts[5].startswith('credentials='):
                    credentials = parts[5].split('=')[1]
                else:
                    credentials = ""
                # Ищем индекс, где начинается часть после ://
                index = l.find('://')
                if index != -1:
                    path = l[index + 3:].strip()
                else:
                    path = ""
                if 'credentials=' in l:
                    credentials = l.split('credentials=')[1].split()[0]
                print(name, credentials, path)
                self.add_row_to_tableview(path, '/media/share/' + name)


    def add_row_to_tableview(self, cell1, cell2):
        model = self.tableView.model()
        # Создаем новую строку с ячейками
        new_row = [QtGui.QStandardItem(cell1), QtGui.QStandardItem(cell2)]
        # Добавляем новую строку в модель
        model.appendRow(new_row)

    def click_checkBox_domain(self):
        if self.checkBox_domain.isChecked():
            self.lineEdit_password.setDisabled(True)
            self.lineEdit_password.setStyleSheet("background-color: #d9d9d9;")
            self.lineEdit_username.setDisabled(True)
            self.lineEdit_username.setStyleSheet("background-color: #d9d9d9;")
            self.lineEdit_domain.setDisabled(True)
            self.lineEdit_domain.setStyleSheet("background-color: #d9d9d9;")
        else:
            self.lineEdit_password.setDisabled(False)
            self.lineEdit_password.setStyleSheet("")
            self.lineEdit_username.setDisabled(False)
            self.lineEdit_username.setStyleSheet("")
            self.lineEdit_domain.setDisabled(False)
            self.lineEdit_domain.setStyleSheet("")

    def button_connect(self):
        self.label_check.clear()
        pattern = r'[?*";^:]'  # запрещенные символы: ? * "
        if not self.lineEdit_path.text():  # если строка пустая
            QtWidgets.QMessageBox.warning(self, "Предупреждение", "Заполните поле Cетевой каталог")
        elif not self.lineEdit_folder.text():
            QtWidgets.QMessageBox.warning(self, "Предупреждение", "Заполните поле Каталог монтирования")
        elif re.search(pattern, self.lineEdit_path.text()):
            QtWidgets.QMessageBox.warning(self, "Предупреждение", "Путь содержит спецсимволы")
        else:
            share_path = self.lineEdit_path.text().replace(r'\\', '/').replace(r'//', '/').replace("\\", '/')
            share_path = trim_trailing_slash('', share_path)
            share_path = "".join(share_path)

            v_folder = self.lineEdit_folder.text()
            v_username = self.lineEdit_username.text()
            v_password = self.lineEdit_password.text()
            v_domain = self.lineEdit_domain.text()
            print(share_path)
            get_host = share_path.split("/", 1)[0]
            print(share_path)
            if ping_host(get_host) == "Хост недоступен!":
                QMessageBox.about(self, 'Connect folder', get_host + ' недоступен!')
                return
            if self.checkBox_smb1.isChecked():
                v_smb1 = 1
            else:
                v_smb1 = 0
            if self.is_edit_mode == False:
                if os.path.isfile(p_folder + '/' + v_folder):
                    QMessageBox.about(self, "About",
                                      "Точка подключения " + '/media/share/' + v_folder + " уже существует.")
                else:
                    # *** Настройка по логину и паролю ***
                    write_auto_master(p_automaster, params_automaster)
                    create_folder(p_folder)
                    auth_path = file_smbuser(v_folder)
                    try:
                        new_file_smbuser = open(auth_path, 'w')
                        new_file_smbuser.write('[smb]\nusername = \npassword = \ndomain = ')
                        new_file_smbuser.close()
                    except PermissionError:
                        print("Отказано в доступе к файлу:", auth_path)
                        QMessageBox.critical(self, "Ошибка", "Отказано в доступе к файлу: " + auth_path)
                        return
                    except FileNotFoundError:
                        print("File not found:", auth_path)
                        return
                    except Exception as e:
                        print("Error:", e)
                        return

                    write_smbuser(auth_path, v_username, v_password, v_domain)  # Запись в /root/.auth_smb/

                    try:
                        with open(p_autosamba, "a") as f:
                            if self.checkBox_domain.isChecked():
                                print("Чекбокс <ПК в домене> активирован")
                                str_params_domain = get_params_domain(share_path, v_folder, v_smb1)
                                f.write(str_params_domain)
                            else:
                                str_credentials = get_params(share_path, v_folder, auth_path, v_smb1)
                                f.write(str_credentials)
                    except PermissionError:
                        print("Отказано в доступе к файлу:", p_autosamba)
                        QMessageBox.critical(self, "Ошибка", "Отказано в доступе к файлу: " + p_autosamba)
                        return
                    except FileNotFoundError:
                        print("File not found:", p_autosamba)
                        return
                    except Exception as e:
                        print("Error:", e)
                        return
            else:
                # *** Настройка по логину и паролю ***
                # *** Запись в /etc/auto.samba ***
                with open(p_autosamba, 'r') as f:
                    data = f.readlines()
                # Запоминаем строку в заданной позиции и удаляем ее из списка
                line = data.pop(line_number)
                auth_path = file_smbuser(v_folder)
                if self.checkBox_domain.isChecked():
                    print("Чекбокс <ПК в домене> активирован")
                    str_credentials = get_params_domain(share_path, v_folder, v_smb1)
                else:
                    str_credentials = get_params(share_path, v_folder, auth_path, v_smb1)
                data.insert(line_number, str_credentials)  # Вставляем нужную строку на ее место
                with open(p_autosamba, 'w') as f:
                    f.write(''.join(data))
                # Запись в /root/.auth_smb/
                write_smbuser(auth_path, v_username, v_password, v_domain)
            add_autofs_to_autostart()
        clear_lineEdit(self)
        self.load_data_tableview()
        self.checkBox_smb1.setChecked(False)

    def button_change_list(self):
        self.label_check.clear()
        self.is_edit_mode = not self.is_edit_mode
        clear_lineEdit(self)

        if self.is_edit_mode:
            self.pushButton_connect.setText("Сохранить изменение")
            self.pushButton_change_list.setText("<-- Вернуться")
            self.pushButton_delete.setVisible(True)

            # Выбираем первую строку, если она существует
            model = self.tableView.model()
            if model.rowCount() > 0:
                self.tableView.selectRow(0)
                # Получаем индекс первой строки и вызываем обработчик клика
                index = model.index(0, 0)  # Индекс первого элемента первой строки
                self.on_tableView_clicked(index)  # Явно вызываем метод обработки клика
        else:
            self.pushButton_connect.setText("Подключить")
            self.pushButton_change_list.setText("Изменить список")
            self.pushButton_delete.setVisible(False)

            # Снимаем выделение строк
            self.tableView.clearSelection()


    def button_delete(self):
        try:
            model = self.tableView.model()
            selected_indexes = self.tableView.selectedIndexes()

            if not selected_indexes:
                return  # Если нет выбранных индексов, выходим из функции

            # Проверяем, что все индексы относятся к одной строке
            first_row = selected_indexes[0].row()
            if all(index.row() == first_row for index in selected_indexes):
                if delete_auth_file and p_autosamba and delete_p_autosamba_folder:
                    os.remove(delete_auth_file)
                    delete_line_by_word(p_autosamba, delete_p_autosamba_folder)
                    model.removeRow(first_row)  # Удаляем первую строку, так как они все относятся к одной и той же строке
                    clear_lineEdit(self)
                    self.delete_auth_file = None
                    self.delete_p_autosamba_folder = None
                    self.p_autosamba = None
                    subprocess.run(["systemctl", "restart", "autofs"])
        except FileNotFoundError:
            print(f"Файл {delete_auth_file} не найден.")


def delete_line_by_word(file_path, word):
    # Открываем файл на чтение и запись
    with open(file_path, 'r+') as f:
        # Читаем все строки файла в список
        lines = f.readlines()
        # Возвращаем указатель в начало файла
        f.seek(0)
        # Проходимся по каждой строке и проверяем первое слово на соответствие
        for line in lines:
            if not line.startswith(word):
                # Если первое слово не соответствует, записываем строку обратно в файл
                f.write(line)
        # Отсекаем лишние символы после последней строки, чтобы файл не увеличивался
        f.truncate()


# Проверка доступаности хоста
def ping_host(host):
    ping_command = os.system(f'ping -c 1 {host} &>/dev/null')
    if ping_command == 0:
        print("Хост доступен!")
    else:
        print("Хост недоступен!")
        ping_result = "Хост недоступен!"
        return ping_result


def get_params(share_path, v_folder, auth_path, v_smb1):
    param_smbver1 = 'vers=1.0' if v_smb1 == 1 else ''
    params_autosamba = ['-fstype=cifs', param_smbver1, 'file_mode=0600', 'dir_mode=0700', 'noperm', 'credentials=' + auth_path]
    params_filtered = [param for param in params_autosamba if param]
    params_string = ','.join(params_filtered)
    str_credentials = v_folder + '  ' + params_string + '  ' + '://' + share_path + '\n'
    return str_credentials


def get_domain_name():
    return os.popen('domainname -d').read().strip()


def get_params_domain(share_path, v_folder, v_smb1):
    str_domain = get_domain_name()
    param_smbver1 = 'vers=1.0' if v_smb1 == 1 else ''
    params_autosamba = ['-fstype=cifs', 'multiuser', 'cruid=$UID', 'sec=krb5', 'iocharset=utf8', param_smbver1,
                        'file_mode=0600', 'dir_mode=0700', 'domain=' + str_domain]
    params_filtered = [param for param in params_autosamba if param]
    params_string = ','.join(params_filtered)
    str_params_domain = v_folder + '  ' + params_string + '  ' + '://' + share_path + '\n'
    return str_params_domain


# Функция поиска и добавления строки в файл
def write_auto_master(file_path, line):
    try:
        with open(file_path, "r") as f:
            lines = f.readlines()
            if line + '\n' in lines:
                return
        # строка не найдена, добавляем в конец файла
        with open(file_path, "a") as f:
            f.write(line + '\n')
    except PermissionError:
        print("Отказано в доступе к файлу:", file_path)
        return
    except FileNotFoundError:
        print("File not found:", file_path)
        return
    except Exception as e:
        print("Error:", e)
        return


def create_folder(p_folder):
    access_rights = 0o755
    try:
        os.mkdir(p_folder, access_rights)
    except FileExistsError:
        pass
    except PermissionError as e:
        print(f"Ошибка доступа: {e}")
        return
    else:
        return True


def file_smbuser(name_folder):
    file_smbuser_path = p_folder + '/' + name_folder
    return file_smbuser_path


# Парсинг auth_path
def write_smbuser(auth_path, v_username, v_password, v_domain):
    config_auth = configparser.ConfigParser()
    config_auth.read(auth_path)
    if 'smb' not in config_auth:
        config_auth.add_section('smb')
    sector_smb = config_auth['smb']
    sector_smb['username'] = v_username
    sector_smb['password'] = v_password
    sector_smb['domain'] = v_domain
    try:
        with open(auth_path, 'w') as configfile:
            config_auth.write(configfile, space_around_delimiters=False)
    except PermissionError:
        print("Permission denied for file:", auth_path)
        return
    except FileNotFoundError:
        print("File not found:", auth_path)
        return
    except Exception as e:
        print("Error:", e)
        return


# autofs
def check_autofs_status():
    try:
        status_output = subprocess.check_output(["systemctl", "is-enabled", "autofs"]).decode().strip()
        return status_output == "enabled"
    except subprocess.CalledProcessError:
        return False


def add_autofs_to_autostart():
    if not check_autofs_status():
        # Add autofs to autostart
        subprocess.run(["systemctl", "enable", "autofs"])
        print("autofs added to autostart.")
    # Restart autofs service
    subprocess.run(["systemctl", "restart", "autofs"])
    print("autofs restarted.")


def clear_lineEdit(self):
    self.lineEdit_domain.clear()
    self.lineEdit_password.clear()
    self.lineEdit_path.clear()
    self.lineEdit_folder.clear()
    self.lineEdit_username.clear()


def get_smbuser(auth_path):
    config_smb = configparser.ConfigParser()
    try:
        config_smb.read(auth_path)
        if config_smb.has_section('smb'):
            get_user = config_smb.get("smb", "username")
            get_password = config_smb.get("smb", "password")
            get_domain = config_smb.get("smb", "domain")
            return get_user, get_password, get_domain
        else:
            print("Section 'smb' not found in auth file")
            return None, None, None
    except (configparser.Error, KeyError) as e:
        print(f"Error reading auth file: {e}")
        return None, None, None


#  Функция обрезки последнего Slash / в конце каталога
def trim_trailing_slash(str_path, str_folder):
    if str_path.endswith('/'):
        str_path = str_path.rstrip('/')
    if str_folder.endswith('/'):
        str_folder = str_folder.rstrip('/')
    return str_path, str_folder


def open_auto_samba():
    if os.path.exists(p_autosamba):
        with open(p_autosamba, 'r') as f:
            lineFile = f.readlines()  # Читаем строки из файла
        return lineFile
    else:
        with open(p_autosamba, "w"):
            pass  # Создаем файл, если он не существует
        return []


def main():
    app = QtWidgets.QApplication(sys.argv)
    window = ConnectfolderApp()
    window.show()
    app.exec_()


if __name__ == '__main__':
    main()
